
package net.sourceforge.asse;

import java.util.*;
import java.sql.*;

// There are two ways to use this facility
// case 1:
//   add command
//   and come back to get result for the jobid.
//   you will block.
//
// case 2:
//   when you add a command, supply a result processor and
//   it will be invoked when there are results.
//   In this case, make sure that the result processor  will be invoked
//   in the executor thread. So do not do GUI manipulation in there.
//

class DBQueue {

    int    next_request = 0;
    int    next_processed = 0;

    int    rowcount; // this is for those who dbexec.
    
    DBMulti dbm; // required to cancel the query.

    Hashtable joblist;
    Hashtable reslist;
    Hashtable resl_list; // result listener

    public DBQueue (DBMulti d) {

	dbm = d;
	joblist = new Hashtable();
	reslist = new Hashtable();
	resl_list = new Hashtable();
    }

    synchronized public boolean cancelJob(Integer jobid) {

	if (reslist.containsKey(jobid)) {
	    reslist.remove(jobid);
	    return true;
	}
	if (joblist.containsKey(jobid)) {
	    joblist.remove (jobid);
	    return true;
	}

	if (jobid.intValue() == next_processed - 1) {
	    try {
		dbm.cancelQuery();
	    } catch (SQLException ex) {
		ex.printStackTrace();
	    }
	    return true;
	}
	return false;
    }

    // Called by the requesting thread. 
    synchronized public Integer addJob( String sql , DBResultListener resp ) {
	Integer jobid = addJob(sql);
	resl_list.put (jobid, resp );
	return jobid;
    }
	
    // Called by the requester thread. 
    synchronized public Integer addJob( String sql ) {
	Integer jobid = new Integer(next_request++);
	joblist.put ( jobid, sql );
	notify();
	return jobid;
    }

    // Called by the executor thread.
    synchronized public Integer getJobId()  
	throws InterruptedException {

	while (joblist.isEmpty())
	    wait();

	return new Integer(next_processed++);
    }

    // Called by the executor thread.
    synchronized public String getJob (Integer jobid) {
	return (String) joblist.remove (jobid);
    }

    // Called by the executor thread.
    synchronized public void addRes ( Integer request, DBQResult result ) {

	// If result processor exists, invoke
	if (resl_list.containsKey(request)) {
	    DBResultListener dbr = 
		(DBResultListener) resl_list.remove(request);
	    dbr.processResult(result);
	} else {
	    // Else put in the queue for requester to come an retrieve.
	    reslist.put ( request, result );
	    notify();
	}
	return;
    }

    // Called by the requester thread. 
    synchronized public DBQResult getRes( Integer jobid ) 
	throws InterruptedException {

	while (!reslist.containsKey (jobid)) 
	    wait();

	return (DBQResult) reslist.remove(jobid);
    }

    // Called by the requester thread. 
    public ezTable[] executeQuery (String sql) 
	throws SQLException {
	
	try {

	    Integer jobid = this.addJob(sql);
	    DBQResult res = this.getRes(jobid);
	  
	    rowcount = res.rowcount;
	    
	    if (res.sqle != null ) {
		throw(res.sqle);
	    }
	    return (res.ezt_array);

	} catch (InterruptedException ie) {
	    ie.printStackTrace();
	}
	return null;
    }
    public int rowcount() {
	return rowcount;
    }

}
