package net.sourceforge.asse;
import javax.swing.*;
import javax.swing.table.*;
import java.text.*;
import java.sql.Date;
import java.awt.Color;
import javax.swing.event.*;
import java.util.Hashtable;
import java.beans.*;
import java.awt.event.*;


class JTableEx extends JTable {

    public JTableEx () {
	super();
 
    }

    public JTableEx (TableModel tm) {
	super(tm);
  	createDefaultRenderers();
	restoreColumnSizes();
	setColumnWidthListeners();
        setColumnHeaderMouseListener();
    }

    public void setModel(TableModel new_model) {

	super.setModel(new_model);
	
	restoreColumnSizes();
	setColumnWidthListeners();
        setColumnHeaderMouseListener();
    }

    // If anyone resized their columns, the sizes should be retained.
    // Here we are assuming that across tables, columns with same name
    // have same characteristics and should have the same width.
    // May be simplistic assumption, but saves us remembering anything
    // more than just column names.

    public void restoreColumnSizes() {
	// This time we look at columns of the JTable instead of 
	// that of TableModel, could be different.

	TableColumnModel tcm;
	tcm = getColumnModel();
	for (int i = 0; i < getColumnCount(); i++) {
	    String colname = getColumnName(i);
	    if (colname != null) {
		Integer n = (Integer)AsseGlobal.colwidths.get(colname);
		if (n != null) {
		    tcm.getColumn(i).setPreferredWidth(n.intValue());
		}
	    }
	}
	sizeColumnsToFit(-1);
    }
    
    public void setSmartEditors (TableInfo ti) {
	for (int i = 0; i < ti.get_columnCount(); i++) {
	    String tname = ti.get_columnType(i+1);
	    //System.out.println("column " + i + " type " + tname);
	    if (tname.indexOf("char") >= 0 ||tname.indexOf("string") >= 0) {
		setSmartColumnEditor(i, ti.get_columnLength(i+1), 
				     SmartTextField.TEXT,
				     0, 0);
		continue;
	    }

	    if (tname.indexOf("int") >= 0) {
		setSmartColumnEditor(i, 15, SmartTextField.NUMBER,
				    15, 0);
		continue;
	    }

	    if (tname.indexOf("float") >= 0) {
		setSmartColumnEditor(i, 50, SmartTextField.NUMBER,
				    50,25);
		continue;
	    }

	    if (tname.indexOf("numeric") >= 0) {
		int prec = ti.get_columnPrecision(i+1);
		int scale = ti.get_columnScale(i+1);

		setSmartColumnEditor(i, prec + scale + 2, SmartTextField.NUMBER,
				    prec,scale);
		continue;
	    }

	    if (tname.indexOf("timestamp") >= 0) {
		setSmartColumnEditor(i, 25, SmartTextField.TEXT, 
				     0, 0);
		continue;
	    }

	    setSmartColumnEditor(i, ti.get_columnLength(i+1),
				 SmartTextField.TEXT, 
				 0, 0);	
	}
    }
    private void setSmartColumnEditor(int col, int length, int type,
				      int precision, int scale) {
	TableColumn aCol = getColumnModel().getColumn(col);
        DefaultCellEditor x = new DefaultCellEditor(new SmartTextField("",type,
								    length,
								    precision,
								    scale));
        x.setClickCountToStart(2);
	aCol.setCellEditor(x);
    }

    protected void createDefaultRenderers() {

	super.createDefaultRenderers();
	setDefaultRenderer(Integer.class, 
                TableCellRendererFactory.get(Integer.class, DwbjProps.getProperty("IntegerFormat","####")));
	setDefaultRenderer(Number.class, 
                TableCellRendererFactory.get(Number.class, DwbjProps.getProperty("DecimalFormat","##.##")));
	setDefaultRenderer(java.util.Date.class, 
                TableCellRendererFactory.get(java.util.Date.class, DwbjProps.getProperty("DateFormat","")));
	setDefaultRenderer(java.sql.Timestamp.class, 
                TableCellRendererFactory.get(java.util.Date.class, DwbjProps.getProperty("DateFormat","")));
        //java.sql.Timestamp.class
    }

    public TableCellRenderer getCellRenderer(int row, int column) {
	TableCellRenderer renderer = super.getCellRenderer(row, column);
        
	try {
	    myJDBCAdapter jadapter = (myJDBCAdapter) dataModel;
            
//            System.out.println("Col:" + column + " " +jadapter.getColumnClass(column).getName());
          //  renderer = get(jadapter.getColumnClass() )
	    JComponent       jc = (JComponent) renderer;
	    Integer rowstat = jadapter.getModStatus(row);
	    if (rowstat == myJDBCAdapter.ROW_INSERTED)
		jc.setBackground(Color.cyan);
	    else if (rowstat == myJDBCAdapter.ROW_MODIFIED) {
		if (jadapter.getModStatus(row, convertColumnIndexToModel(column)) == myJDBCAdapter.MODIFIED)
		    jc.setBackground(Color.red);
		else
		    jc.setBackground(Color.lightGray);
	    } else 
		jc.setBackground(Color.white);
	} catch (Exception ex) {
	    ;
	}

	return renderer;

    };

    public boolean isCellEditable(int row, int col) {
	//System.out.println("Checking if editable.");
	try {
	    myJDBCAdapter jadapter = (myJDBCAdapter) dataModel;
	    return jadapter.isCellEditable(row, convertColumnIndexToModel(col));
	} catch (Exception ex) {
	    ex.printStackTrace();
	    return false;
	}
    }	

    private void setColumnWidthListeners() {

	TableColumnModel tcm = getColumnModel();

	for (int i = 0; i < tcm.getColumnCount(); i++) {
	    TableColumn col = tcm.getColumn(i);
	    final String colname = getColumnName(i);
	    col.addPropertyChangeListener( new PropertyChangeListener() {
		    public void propertyChange (PropertyChangeEvent evt) {
			if (evt.getPropertyName().compareTo("width") == 0) {
			    if (colname.indexOf("col-") != 0)
				AsseGlobal.colwidths.put (colname,
						     evt.getNewValue());
			}
		    }
		} );
	}
    }
    
    private void setColumnHeaderMouseListener() {
        final JTable jt = this;
        JTableHeader theader = getTableHeader(); 
        if (theader == null)
            return;
        
        jt.setColumnSelectionAllowed(false); 
        MouseAdapter listMouseListener = new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                TableColumnModel columnModel = jt.getColumnModel();
                int viewColumn = columnModel.getColumnIndexAtX(e.getX());
                int column = jt.convertColumnIndexToModel(viewColumn);
                if(e.getClickCount() == 1 && column != -1) {
                    if (e.getButton() == MouseEvent.BUTTON1) {
                        //System.out.println("Sorting ...");
                        int shiftPressed = e.getModifiers()&InputEvent.SHIFT_MASK;
                        boolean ascending = (shiftPressed == 0);
                        if (jt.getModel() instanceof myJDBCAdapter) {
                            myJDBCAdapter jda = (myJDBCAdapter) jt.getModel();
                            jda.sort(column, ascending);
                        }
                    } 
                }
            }
        };

        theader.addMouseListener(listMouseListener); 
    }
//    protected boolean processKeyBinding(KeyStroke ks,
//                                        java.awt.event.KeyEvent e,
//                                        int condition,
//                                        boolean pressed) {
//      
//        if( ((e.getModifiersEx() & e.ALT_DOWN_MASK) == e.ALT_DOWN_MASK) ||
//            ((e.getModifiersEx() & e.CTRL_DOWN_MASK) == e.CTRL_DOWN_MASK) ||
//            ((e.getModifiersEx() & e.SHIFT_DOWN_MASK) == e.SHIFT_DOWN_MASK) 
//          )
//        {
//            return false;
//        }
//        return super.processKeyBinding(ks, e, condition, pressed);
//    }
    
}
