package net.sourceforge.asse;
/* (swing1.1beta3) */
//package jp.gr.java_conf.tame.swing.examples;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;

// These are printing related
import java.awt.print.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.Dimension;


/**
 * @version 1.0 11/09/98
 */

class RowHeaderRenderer extends JLabel implements ListCellRenderer {
    
    
    RowHeaderRenderer(JTable table) {
        JTableHeader header = table.getTableHeader();
        setOpaque(true);
        setBorder(UIManager.getBorder("TableHeader.cellBorder"));
        setHorizontalAlignment(CENTER);
        setForeground(header.getForeground());
        setBackground(header.getBackground());
        setFont(header.getFont());
    }
    
    public Component getListCellRendererComponent(	   JList list,
    Object value,
    int index, boolean isSelected, boolean cellHasFocus) {
        setText((value == null) ? "" : value.toString());
        return this;
    }
}

public class JTableRowHeader extends JTableEx implements Printable {
    
    JScrollPane _scroll;
    JList _rowHeader;
    
    public JTableRowHeader(TableModel tm) {
        super(tm);
        setup();
    }
    
    public JTableRowHeader() {
        super();
        setup();
    }
    
    private void setup() {
        ListModel lm = new AbstractListModel() {
            public int getSize() { return getModel().getRowCount(); }
            public Object getElementAt(int index) {
                return new Integer(index + 1);
            }
        };
        
        setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        
        _rowHeader = new JList(lm);
        _rowHeader.setFixedCellWidth(50);
        
        //This cludge for 1.3 and 1.2 difference.
        if (System.getProperty("java.specification.version").compareTo("1.2") == 0)
            _rowHeader.setFixedCellHeight( getRowHeight() +
            getRowMargin()
            // + getIntercellSpacing().height
            );
        else
            _rowHeader.setFixedCellHeight( getRowHeight() );
        
        _rowHeader.setCellRenderer(new RowHeaderRenderer(this));
        _rowHeader.setBackground(getTableHeader().getBackground());
        
        _scroll = new JScrollPane( this );
        _scroll.setRowHeaderView(_rowHeader);
    }
    
    JScrollPane getScrollPane() {
        return _scroll;
    }
    
    public int print(Graphics g, PageFormat pageFormat,
    int pageIndex) throws PrinterException {
        Graphics2D  g2 = (Graphics2D) g;
        g2.setColor(Color.black);
        int fontHeight=g2.getFontMetrics().getHeight();
        int fontDesent=g2.getFontMetrics().getDescent();
        
        //leave room for page number
        double pageHeight = pageFormat.getImageableHeight()-fontHeight;
        double pageWidth = pageFormat.getImageableWidth();
        double tableWidth =
        (double)this.getColumnModel().getTotalColumnWidth() +
        _rowHeader.getFixedCellWidth();
        
        double scale = 1;
        if (tableWidth >= pageWidth) {
            scale =  pageWidth / tableWidth;
        }
        
        double headerHeightOnPage=
        this.getTableHeader().getHeight()*scale;
        double tableWidthOnPage=tableWidth*scale;
        
        double oneRowHeight=(this.getRowHeight()+
        this.getRowMargin())*scale;
        int numRowsOnAPage=
        (int)((pageHeight-headerHeightOnPage)/oneRowHeight);
        double pageHeightForTable=oneRowHeight*numRowsOnAPage;
        int totalNumPages= (int)Math.ceil((
        (double)this.getRowCount())/numRowsOnAPage);
        
        double rowHdrWidth = _rowHeader.getFixedCellWidth() * scale;
        
        if(pageIndex>=totalNumPages) {
            return Printable.NO_SUCH_PAGE;
        }
        
        g2.translate(pageFormat.getImageableX(),
        pageFormat.getImageableY());
        g2.drawString("Page: "+(pageIndex+1) + "/" + totalNumPages,
        (int)pageWidth/2-35,
        (int)(pageHeight+fontHeight-fontDesent));//bottom center
        
        
        g2.translate(0f,headerHeightOnPage);
        g2.translate(0f,-pageIndex*pageHeightForTable);
        
        if (pageIndex + 1 == totalNumPages) {
            int lastRowPrinted = numRowsOnAPage * pageIndex;
            int numRowsLeft = this.getRowCount() - lastRowPrinted;
            g2.setClip(0, (int)(pageHeightForTable * pageIndex),
            (int) Math.ceil(rowHdrWidth),
            (int) Math.ceil(oneRowHeight * numRowsLeft));
        }
        //else clip to the entire area available.
        else{
            g2.setClip(0, (int)(pageHeightForTable*pageIndex),
            (int) Math.ceil(rowHdrWidth),
            (int) Math.ceil(pageHeightForTable));
            
        }
        
        g2.scale(scale,scale);
        _rowHeader.paint(g2);
        g2.scale(1/scale,1/scale);
        
        g2.translate(rowHdrWidth,0f);
        
        //If this piece of the table is smaller than the size available,
        //clip to the appropriate bounds.
        if (pageIndex + 1 == totalNumPages) {
            int lastRowPrinted = numRowsOnAPage * pageIndex;
            int numRowsLeft = this.getRowCount() - lastRowPrinted;
            g2.setClip(0,
            (int)(pageHeightForTable * pageIndex),
            (int) Math.ceil(tableWidthOnPage -
            rowHdrWidth),
            (int) Math.ceil(oneRowHeight * numRowsLeft));
        }
        //else clip to the entire area available.
        else{
            g2.setClip(0,
            (int)(pageHeightForTable*pageIndex),
            (int) Math.ceil(tableWidthOnPage -
            rowHdrWidth),
            (int) Math.ceil(pageHeightForTable));
        }
        
        g2.scale(scale,scale);
        this.paint(g2);
        g2.scale(1/scale,1/scale);
        
        g2.translate(0f,pageIndex*pageHeightForTable);
        g2.translate(0f, -headerHeightOnPage);
        g2.setClip(0, 0,
        (int)Math.ceil(tableWidthOnPage - rowHdrWidth),
        (int)Math.ceil(headerHeightOnPage));
        g2.scale(scale,scale);
        this.getTableHeader().paint(g2);//paint header at top
        
        return Printable.PAGE_EXISTS;
    }
    
    public static void main(String[] args) {
        JFrame frame = new myframe("JTableRowHeader");
        JTableRowHeader  table = new JTableRowHeader(new ezTable(20, 5));
        frame.getContentPane().add( table.getScrollPane());
        frame.setSize(500,250);
        frame.setVisible(true);
    }
    
}

