package net.sourceforge.asse;

import java.io.*; 

//  This class creates log files for SYSTEM.OUT stuff in the temporary
//  directory. The files are not deleted on exit. However, log files older than a 
//  day are deleted.

class LogFiles {

    public LogFiles(String prefix) {

	// 	 Lets make sure that there are not more than 100 files
	// 	 with this signature already.

	try {

	    File outf = File.createTempFile(prefix,".log");

	    String outfname = outf.getAbsolutePath();
	    String tmp_dir_name =  outfname.substring(0, outfname.lastIndexOf(prefix));
	    File tmp_files = new File(tmp_dir_name);
	    String[] logfile_names = tmp_files.list(new LogFileFilter(prefix));
	    if (logfile_names == null) {
// 		System.out.println("No old files found.");
// 		System.exit(1);
		return;
	    }	  
	    
	    long current_time = outf.lastModified();
	    for (int i = 0; i < logfile_names.length; i++) {
		File fr = new File(tmp_dir_name + logfile_names[i]);
		if (current_time - fr.lastModified() > 24 * 3600 * 1000) {
		    System.out.println("removing more  than a day old log file: " + tmp_dir_name + logfile_names[i]);
		    fr.delete();
		}
	    }

	    System.out.println("**Logging all errors, stdout  to log file**: " + outf.getAbsolutePath());
	    PrintStream prnf = new PrintStream(new FileOutputStream(outf));
	    System.setOut(prnf);
	    System.setErr(prnf);

	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    public static void main (String args[]) {

	new LogFiles(args[0]);
    }

    class LogFileFilter implements FilenameFilter {

	String prefix;
	public LogFileFilter(String pref) {
	    prefix = pref;
	}

	public boolean accept (File f, String n) {
	    // We are just checking that the file name is like
	    // dwbj*log
		    
            if (n.length() < 7)
		return false;
	    if (n.substring(0,4).compareTo(prefix) == 0 &&
		(n.length() - n.lastIndexOf("log")) == 3)
		return true;
	    else 
		return false;
	}
    }
	
}

