package net.sourceforge.asse;

import java.awt.*;
import java.awt.print.*;
import java.util.*;
import java.io.*;

public class PrintAnyText implements Printable {
    
    Vector      _lines;
    int         _lines_perpage = 0;
    int         _page_count = 0;
    PageFormat  _curPageFormat;
    Font        _font = new Font ("TimesRoman", Font.PLAIN, 12);
    int         _lineh = _font.getSize ();
  
    public PrintAnyText (String text) {
	
	_lines = new Vector();
	
	StringTokenizer st = new StringTokenizer (text, "\n");
	while (st.hasMoreTokens ()) {
	    String line = st.nextToken ();
	    _lines.add(line);
	}


	PrinterJob pj = PrinterJob.getPrinterJob();
	PageFormat pf = pj.pageDialog(pj.defaultPage());
	
	pj.setPrintable(this, pf);
	if (pj.printDialog()) {
	    try { pj.print(); }
	    catch (PrinterException pe) {
		System.out.println(pe);
	    }
	}
    }

    public int print (Graphics g, PageFormat pf, int idx)
	throws PrinterException {

	g.setFont (_font);
     	int fontHeight=g.getFontMetrics().getHeight();
     	int fontDesent=g.getFontMetrics().getDescent();

	_lineh = fontHeight;

	int pageHeight = (int) pf.getImageableHeight () - _lineh;
	_page_count = (int)Math.ceil((double)_lines.size() /(pageHeight/_lineh) );
	_lines_perpage = (int)(pageHeight /_lineh) - 1;

	if (idx >= _page_count)
	    return Printable.NO_SUCH_PAGE;

	renderPage (g, pf, idx);

	return Printable.PAGE_EXISTS;
    }
	
    void renderPage (Graphics g, PageFormat pf, int idx) {
	
	int xo = (int) pf.getImageableX ();
	int yo = (int) pf.getImageableY ();
	int y  = _lineh;
	
	int lines_index = idx * _lines_perpage;
	for (int line_no = 0 ; 
	     line_no < _lines_perpage;
	     line_no++) {

	    if (lines_index >= _lines.size())
		break;
	    
	    String line = (String) _lines.get(lines_index++);
	    g.drawString (line, xo, y + yo);
	    y += _lineh;
	}
	g.drawString("Page: "+(idx + 1) + "/" + _page_count,
		     xo + (int)pf.getImageableWidth()/2-35,
		     y +  yo);
	
    }


    public  static void main(String args[]) {

	try {
	    
	    StringBuffer toprint = new StringBuffer();
	    BufferedReader br = new BufferedReader(new FileReader("PrintAnyText.java"));
	    String nextline;
	    while ((nextline = br.readLine()) != null) {
		toprint.append(nextline);
		toprint.append("\n");
	    }
	    
	    new PrintAnyText(toprint.toString());
	    
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

}









