package net.sourceforge.asse;

import java.awt.Toolkit; 
import javax.swing.*; 
import javax.swing.text.*; 


// A TextField with available restrictions on 
//  a) length 
//  b) input type - number only, anything else

public class SmartTextField extends JTextField { 
    public final static int NUMBER = 1;
    public final static int TEXT   = 2;
    // Exact numerics need precision & scale to be defined
    // And allow optional leading +/-  sign.

    public SmartTextField(int max_length) { 
	this(TEXT, max_length); 
    } 

    // Use this constructor only for TEXT types
    public SmartTextField( int type, int max_length ) { 
	this(null, type, max_length, 0, 0); } 
    
    // For numbers, always use this.
    public SmartTextField(String text, int type, 
			  int max_length, int precision, int scale ) { 
	super(new SmartPlainDocument(type, max_length, precision, scale), 
	      text, 10 ); 
    } 

    public static void main (String args[]) {
	
	JFrame f = new myframe("SmartTextField");
	JPanel  p = new JPanel();
	p.add(new SmartTextField(SmartTextField.TEXT, 10));
	p.add(new SmartTextField("", SmartTextField.NUMBER,
				 22, 20, 5));
	p.add(new SmartTextField("", SmartTextField.NUMBER,
				 12, 10, 5));
	f.getContentPane().add(p);
	f.setSize(500, 500);
	f.setVisible(true);
    }
} 

// An extension of PlainDocument that restricts the length of the content 
// it contains. 

class SmartPlainDocument extends PlainDocument { 
    
    public SmartPlainDocument (int dtype, int maxl, 
			       int pprec, int pscale) {
	datatype = dtype;
	maxlength = maxl;
	precision = pprec;
	scale = pscale;
    }

    // If this insertion would exceed the maximum document length,
    // we "beep" and do  nothing else. 
    // Otherwise, super.insertString() is called. 
    public void insertString(int offset, String str, AttributeSet a) 
	throws BadLocationException { 

	String s = getText(0,offset) + str;
	if (offset < getLength()) {
	    s += getText(offset, getLength() - offset);
	}

	DOIT: 
	{
	    if (rtrim(s).length() > maxlength) {
		Toolkit.getDefaultToolkit().beep(); 
		return;
	    }
	    
	    if (datatype == SmartTextField.TEXT) 
		break DOIT; // no other checks for text field
		
	    if (s.compareTo("-") == 0)
		break DOIT;

	    boolean sign_in_front = false;
	    // remove leading -/+ sign, if any
	    if (s.length() > 0) {
		char ch1 = s.charAt(0);
		if (ch1 == '-' || ch1 == '+')
		    s = s.substring(1);
	    }

	    if (s.indexOf('.') != s.lastIndexOf('.')) {
		Toolkit.getDefaultToolkit().beep(); 
		return;
	    }

	    // lets make sure all are digits
	    char[] sarr = (s.replace('.', '1')).toCharArray();
	    for (int i = 0; i < sarr.length; i++) {
		if (!Character.isDigit(sarr[i])) {
		    Toolkit.getDefaultToolkit().beep(); 
		    return;
		}
	    }
	    
	    String sfore, saft;
	    int    decimal_pos = s.indexOf('.');
	    if (decimal_pos >= 0) {
		sfore = s.substring(0, decimal_pos);
		saft  = s.substring(decimal_pos + 1);
	    } else {
		sfore = s;
		saft  = "";
	    }
	    
	    if (precision - scale < sfore.length()) {
		Toolkit.getDefaultToolkit().beep(); 
		return;
	    }
	    if (scale < saft.length()) {
		Toolkit.getDefaultToolkit().beep(); 
		return;
	    }
	} 
	super.insertString(offset, str, a); 
    }

    private String rtrim(String s) {
	return  s.substring(0, s.indexOf(s.trim()) + s.trim().length());
    }

    private int precision; //total number of digits (on either side of .)
    private int scale; //total number of digits (to the right of .)
    private int maxlength; 
    private int datatype;
}


