package net.sourceforge.asse;

import java.util.Vector;

// This class provides the following :
// -  storage for SQL and associated Results
// -  ability to mark any as locked/unlocked (default)
// -  provide unique tags for each (for a drop down list)
// -  persistence (for the whole list).
//
class SqlNResults {

    final int MAX_TAG_LENGTH = 35;
    final int MAX_HISTORY    = 10; //maximum unlocked queries in history
    class sqlnr {
	public final String sql;
	public final ezTable[] res;
	boolean locked = false;
        String tag;
	public sqlnr (String s, ezTable[] r, String ntag) {
	    sql = s;
	    res = r;
	    tag = ntag;
	}
	
	public  String tag() {
	    if (locked) 
		return "*" + tag;
	    else
		return " " + tag;
	}

	public void set_locked(boolean set) {
	    locked = set;
	}
	
	public boolean is_locked() {
	    return locked;
	}
	
	public boolean match_tag(String ntag) {
	    if (ntag.length() < 2)
		return false;
	    return (ntag.substring(1).compareTo(tag) == 0);
	}
    }

    Vector sv = new Vector();
    int    seq_counter = 0;

    public String add ( String s, ezTable[] r) {
	
	if (r == null || r.length == 0) {
	    return null;
	}

	int unlocked = 0;
	// Count how many unlocked results we have
	for (int i = 0; i < sv.size() -1 ; i++) {
	    sqlnr x1 = (sqlnr)sv.get(i);
	    if (!x1.is_locked())
		unlocked++;
	}
	while (unlocked > MAX_HISTORY) {
	    for (int i = 0; i < sv.size() -1 ; i++) {
		sqlnr x1 = (sqlnr)sv.get(i);
		if (!x1.is_locked()) {
		    sv.removeElementAt(i);
		    unlocked--;
		    break;
		}
	    }
	}
	    
		
	seq_counter++;
	String tmp_tag = 
	    Integer.toString(seq_counter) + ' ' + s.replace('\n', ' ');
	if (tmp_tag.length() > MAX_TAG_LENGTH)
	    tmp_tag = tmp_tag.substring(0, MAX_TAG_LENGTH - 1);
	
	sqlnr x = new sqlnr(s, r, tmp_tag);
	sv.add(x);
	return x.tag();
    }
	
    public boolean set_lock(String tag, boolean set) {

	for (int i = 0; i < sv.size(); i++) {
	    sqlnr x = (sqlnr) sv.get(i);
	    if (x.match_tag(tag)) {
		x.set_locked(set);
		return true;
	    }
	}
	return false;
    }

    public ezTable[] get_res(String tag) {

	for (int i = 0; i < sv.size(); i++) {
	    sqlnr x = (sqlnr) sv.get(i);
	    if (x.match_tag(tag)) {
		return x.res;
	    }
	}
	return null;
    }
    public String get_sql(String tag) {

	for (int i = 0; i < sv.size(); i++) {
	    sqlnr x = (sqlnr) sv.get(i);
	    if (x.match_tag(tag)) {
		return x.sql;
	    }
	}
	return "";
    }

    public boolean is_locked(String tag) {

	for (int i = 0; i < sv.size(); i++) {
	    sqlnr x = (sqlnr) sv.get(i);
	    if (x.match_tag(tag)) {
		return x.is_locked();
	    }
	}
	return false;
    }

    public String get_tag(String ntag) {

	for (int i = 0; i < sv.size(); i++) {
	    sqlnr x = (sqlnr) sv.get(i);
	    if (x.match_tag(ntag)) {
		return x.tag();
	    }
	}
	return "not_found";
    }


    String [] get_tags(boolean reverse) {
	
	int start;
	int increment;
	String[] tags = new String[sv.size()];

	if (reverse) {
	    start = sv.size() -1;
	    increment = -1;
	}
	else {
	    start = 0;
	    increment = 1;
	}

	int tag_index = 0;
	for (int i = start; i >= 0; i += increment) {
	    sqlnr x1 = (sqlnr)sv.get(i);
	    tags[tag_index++] = x1.tag();
	}
	
	return tags;
    }
}
