
package net.sourceforge.asse;



import java.util.*;
import javax.swing.JOptionPane;
import javax.swing.JOptionPane.*;
import java.sql.*;

class TableInfo {

    static Hashtable s_tables = new Hashtable();

    public static TableInfo get_cached_table(String tablename) {
	Object o1 = s_tables.get(tablename);
	if (o1 != null) 
	    return ((TableInfo)o1);
	else
	    return null;
    }

	
    private void set_cached_table(String tablename, TableInfo table) {
	s_tables.put(tablename, table);
    }

    String  full_name;
    String  table_name;
    String  owner_name;
    String  db_name;

    ezTable index_info;
    ezTable column_info;
    // These are supported base types. 
    // Anything else will not work.
    private static  Hashtable  base_types;

    private static void load_base_types() {

	base_types = new Hashtable();
	base_types.put ("int", "int");
	base_types.put ("bit", "boolean");
	base_types.put("tinyint","int");
	base_types.put("image","binary");  //unsupported
	base_types.put("timestamp","binary"); //unsupported
	base_types.put("varbinary","binary"); //unsupported
	base_types.put("binary","binary"); //unsupported
	base_types.put("text","binary"); //unsupported for edit
	base_types.put("char","string");
	base_types.put("nchar","string");
	base_types.put("numeric","numeric");
	base_types.put("decimal","numeric");
	base_types.put("money","float");
	base_types.put("smallmoney","float");
	base_types.put("int","int");
	base_types.put("smallint","int");
	base_types.put("float","float");
	base_types.put("real","float");
	base_types.put("datetime","timestamp");
	base_types.put("smalldatetime","timestamp");
	base_types.put("nvarchar","string");
	base_types.put("sysname","string");
	base_types.put("varchar","string");

    }

    boolean valid = false;

    public boolean isValid() {
//  	System.out.println("Checking validity, is : " + (valid ? "true" : "false"));
	return valid;
    }

    public String getTableName() {
	return full_name;
    }

    // Expected table name is be db.user.table 
    public TableInfo (String p_db,
		      String p_owner,
		      String p_table,
		      ezTable[] res //result of sp_help 
		      )  {
	
	if (base_types == null) {
	    load_base_types();
	}
	db_name    = p_db;
	owner_name = p_owner;
	table_name = p_table;
	
	if (res.length < 1) 
	    return;
	
	for (int i = res.length -1; i >= 0; i--) {
	    int colid = res[i].findColumn("index_name");
	    if ( colid >= 0) {
		index_info = res[i];
	    }
	    colid = res[i].findColumn("Column_name");
	    if ( colid >= 0) {
		column_info = res[i];
		// change the Type information if it contains
		// any user defined types
		for (int j1 = 0; j1 < column_info.getRowCount(); j1++) {
		    String utype = column_info.getString(j1,"Type");
		    String ctype = (String)base_types.get(utype.trim());
		    if (ctype == null) {
			String ntype;
			ntype = xlate_user_type(db_name, utype);
			//  			    System.out.println("user type:" + utype  +
			//  					       " changed to:" + ntype);
			column_info.setValueAt(ntype, j1, 
					       column_info.findColumn("Type"));
		    } else
			column_info.setValueAt(ctype, j1, 
					       column_info.findColumn("Type"));
		    
		    }
	    }
	    valid = true;
	}
	
	full_name = db_name + "." + owner_name + "." + table_name;
	
	// set editable flag.
	if (!checkIfEditable()) {
	    System.out.println("Table not editable\n" +
			       " since could not find any unique index \n" +
			       " that does not contain binary/text/image columns.\n");
	}
	
	set_cached_table(full_name, this);
	
    } 

    // Queries on columns
    // Names, types, if identity
    public int get_columnCount() {
	if (!valid) {
	    return 0;
	}
	return column_info.getRowCount();
    }
    // colid of the table goes from 1...n
    public String get_columnName(int colid) {
	if (!valid || colid < 1 || colid > column_info.getRowCount()) {
	    return "";
	}
	return column_info.getString(colid - 1,"Column_name").trim();
    }				

    public int get_columnId(String colname) {
	for (int i = 0; i < column_info.getRowCount(); i++) {
	    if (column_info.getString(i, "Column_name").trim().compareTo(colname) == 0) 
		return i + 1;
	}
	System.out.println("supplied invalid column name in get_columnId:" +
			   colname);
	return -2;
    }
    public String[] get_columnTypes() {
	String[] coltypes = new String [ column_info.getRowCount() ];
	
	for (int i = 0; i < coltypes.length; i++) 
	    coltypes[i] = get_columnType(i + 1);

	return coltypes;
    }
    
    public String get_columnType(int colid) {
	if (!valid || colid < 1 || colid > column_info.getRowCount()) {
	    return "";
	}
	String base_type = column_info.getString(colid - 1,"Type");
	if (base_type.length() == 0 || base_type.compareTo("binary") == 0) 
	    System.out.println("this type is unsupported for edits." + 
			       base_type);
	return base_type;
    }
    public int get_columnLength(int colid) {
	if (!valid || colid < 1 || colid > column_info.getRowCount()) {
	    return 0;
	}
	return column_info.getInt(colid - 1, "Length");
    }

    public int get_columnPrecision(int colid) {
	if (!valid || colid < 1 || colid > column_info.getRowCount()) {
	    return 0;
	}
	return column_info.getInt(colid - 1, "Prec");
    }


    public int get_columnScale(int colid) {
	if (!valid || colid < 1 || colid > column_info.getRowCount()) {
	    return 0;
	}
	return column_info.getInt(colid - 1, "Scale");
    }

    public boolean is_identity(String colname) {
	if (!valid) {
	    return false;
	}
	for (int i = column_info.getRowCount() - 1;
	     i >= 0; i--) {
	    
	    if (column_info.getString(i, "Column_name").trim().compareTo(colname) == 0) {
		if (column_info.getInt(i, "Identity") == 1) 
		    return true;
		else
		    return false;
	    }
	}
	return false;
    }
	
    public int get_identity() {
	if (!valid) {
	    return -1;
	}
	for (int i = column_info.getRowCount() - 1;
	     i >= 0; i--) {
	    if (column_info.getInt(i, "Identity") == 1) 
		return i;
	}
	return -2;
    }
    // Queries on indexes
    // How many unique
    // What are key columns for a given index.
    public String[] get_uniqueIndexes() {
	Vector ind_vec = new Vector();
	if (index_info != null) {
	    for (int i = index_info.getRowCount() - 1; i >=0; i--) {
		String desc;
		desc = index_info.getString(i, "index_description");
		if (desc.indexOf("unique") >= 0) {
		    ind_vec.addElement(index_info.getString(i, "index_name"));
		}
	    }       
	}

	String [] index_list = new String [ ind_vec.size() ];
	ind_vec.copyInto(index_list);

	return index_list;
    }

    private boolean checkIfEditable() {
	// Lets determine if this table is editable
	// via unique indexes
	tableEditable = false;

	String[] uniqind = get_uniqueIndexes();
	for (int idx = 0; idx < uniqind.length; idx++) {

	    String[] keys = get_indexKeys(uniqind[idx]);
	    boolean flag_contains_binary = false;
	    for (int keyid = 0; keyid < keys.length; keyid++) {
		String coltype =  get_columnType(get_columnId(keys[keyid]));
		if (coltype.indexOf("binary") >= 0) {
		    flag_contains_binary = true;
		    break;
		}
	    }
	    if (!flag_contains_binary) {
		tableEditable = true;
		return true;
	    }
	}
	return tableEditable;
    }
	
    public boolean isTableEditable() {
	return tableEditable;
    }
	
    public String[] get_indexKeys(String ind_name) {
	Vector key_vec = new Vector();
	String [] key_list;

	boolean notfound = true;
	for (int i = index_info.getRowCount() - 1; i >=0 && notfound; i--) {
	    String name;
	    name = index_info.getString(i, "index_name");
	    if (name.equals(ind_name)) {
		StringTokenizer st = new StringTokenizer(index_info.getString(i, "index_keys"),
							 " ,\t\n\r\f");
		while (st.hasMoreTokens()) {
		    key_vec.addElement(st.nextToken());
		}
		notfound = false;
		continue;
	    }
	}
        key_list = new String [ key_vec.size() ];
	key_vec.copyInto(key_list);
	return key_list; 	
    }

    public void  print() {
	System.out.println("*** printing out *** : " + full_name);
	String []s = get_uniqueIndexes();
	for (int j=0; j < s.length; j++) {
	    System.out.println("--unique index : " + s[j]);
	    String keylist [];

	    keylist = get_indexKeys(s[j]); 		    
	    System.out.println("keys are :"); 
	    for (int k=0; k < keylist.length; k++) {
		System.out.println("    " + keylist[k]);
	    }
	}
	for (int i1 = 1; i1 <= column_info.getRowCount(); i1++) {
	    System.out.println(" Column name " + 
			       get_columnName(i1) +
			       " Column type " + 
			       get_columnType(i1) +
			       " editable : " + isColEditable(i1 - 1) );
	}
	System.out.println("*** Finished printing *** : " + full_name);
    }	


    public static void main  (String args[]) {
	
	try {
	    DBMulti dbh = new DBMulti("");
	    dbh.set_debug(true);
	    ezTable[] res = dbh.executeQuery("sp_help 'dev.dbo.test'");
	    TableInfo tbli = new TableInfo("dev", "dbo",
					   "test", res);
	    if (tbli.isValid()) {
		tbli.print();
	    } else 
		System.out.println( " is invalid.");
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }
    
    // This function loads all datatype information in this
    // database. This is required to handle user defined
    // datatypes.
    String xlate_user_type (String db_name, String user_type) {
	
	String base_type;
	int    base_type_length;

	String cmd = db_name + "..sp_help " + user_type;

	if (base_types.get(cmd) != null) 
	    return (String) base_types.get(cmd);

	try {
	    ezTable res[] = asse.dbq.executeQuery (cmd);

	    if (res.length != 1) {
		System.out.println("Can't do :" + cmd);
		System.exit(1);
	    }
	    base_type        = res[0].getString(0, "Storage_type");
	    base_type_length = res[0].getInt(0, "Length");
	}
	
	catch (Exception ex) {
	    System.out.println("Error executing sp_datatype_info");
	    ex.printStackTrace();
	    System.exit(1);
	    base_type = "";
	}
	base_types.put(cmd, base_type);
	return base_type;
    }

    public boolean isColEditable(int column) {
	boolean [] x;
	x = getEditable();

 	return x[column];
    }

    public boolean [] getEditable() {
	if (isEditable != null)
	    return isEditable;

	isEditable = new boolean[get_columnCount()];
	for (int i=0; i < get_columnCount(); i++) {
	    String ctype = get_columnType(i+1);
	    if (ctype.length() == 0  || ctype.compareTo("binary") == 0) {
		System.out.println("columns " + i + " is not editable: " +
				   ctype);
		isEditable[i] = false;
	    } else
		isEditable[i] = true;
	}
	int ident = get_identity();
	if (ident >= 0) 
	    isEditable[ident] = false;

	return isEditable;
    }

    boolean [] isEditable;
    boolean   tableEditable = false;

}

