package net.sourceforge.asse;


/**
 * An adaptor, transforming the JDBC interface to the TableModel interface.
 *
 * @version 1.20 09/25/97
 * @author Philip Milne
 */

import java.util.Vector;
import java.sql.*;
import javax.swing.table.*;
import javax.swing.*;
import javax.swing.event.TableModelEvent;
import java.text.NumberFormat; 
import java.util.StringTokenizer;

public class myJDBCAdapter extends AbstractTableModel {
    
    ezTable          ezt;
    ezTable          ezt_orig;
    ezTable          ezt_deleted; // only to hold delted data

    Boolean[]           colModifiable;
    String              pkTable;  // Primary Modifiable Table
    String[]            pkColumns; // Primary Keys for pkTable, by name
    boolean[]           ispkColumn; // Array indicating True 
    // if the column is a primary
//     boolean[][]      modified;
//     int[]            modifiedrow;
    ezTable             modified;
    Vector              modifiedrow;

    Class               colClass[];
    int                 identity_column = -1;

    public static final Integer
	MODIFIED       = new Integer(1),
	UNMODIFIED     = new Integer(0);
    
    public static final Integer
	ROW_UNMODIFIED = new Integer(0),
	ROW_INSERTED   = new Integer(1),
	ROW_MODIFIED   = new Integer(2),
	ROW_DELETED    = new Integer(3);


    public myJDBCAdapter (myJDBCAdapter old) {
	this(new ezTable(old.ezt));
    }

    public myJDBCAdapter(ezTable pezt ) {
	try {
	    ezt      = pezt;
            setOtherezTables();
	    setNotModified();
	    setColumnClass(ezt.getColTypeNames());
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }
   
    public void setTableInfo( TableInfo t ) {
	
	if (t.isTableEditable()) {
	    setEditable(t.getEditable());
	} else {
	    setEditable(null);
	}
	setPrimaryTable(t.getTableName());
	set_identity_column(t.get_identity());

	String[] indexes = t.get_uniqueIndexes();
	if (indexes.length < 1) {
	    System.out.println("No unique indexes found.");
	    setPrimaryKey(new String[0]);	
	} else {
	    System.out.println("Setting unique indexes.");
	    String[] ucolumns = t.get_indexKeys(indexes[0]);
	    setPrimaryKey(ucolumns);	
	}
	setColumnClass(t.get_columnTypes());

	//debugPrimaryKeys("setTableInfo");

    }
 
    private void setColumnClass(String [] coltypes) {
	
	colClass = new Class [ coltypes.length ];
	for (int i=0; i < coltypes.length; i++) {
	    
	    String tn = coltypes[i];

	    if (tn.indexOf("char") >=0 || 
		tn.indexOf("text") >= 0 ||
		tn.indexOf("sysname") >= 0)
		colClass[i] =  String.class;
	    else if (tn.indexOf("bit") >= 0) 
		colClass[i] =  Boolean.class;
	    else if (tn.indexOf("int") >= 0)
		colClass[i] =  Integer.class;
	    else if (tn.indexOf("float") >= 0 || 
		     tn.indexOf("double") >= 0 || 
		     tn.indexOf("real") >= 0 || 
		     tn.indexOf("numeric") >= 0 || 
		     tn.indexOf("decimal") >= 0 || 
		     tn.indexOf("money") >= 0)
		colClass[i] =  Number.class;
	    else  if (tn.indexOf("timestamp") >= 0 ) 
		colClass[i] = java.sql.Timestamp.class;
	    else  if (tn.indexOf("datetime") >= 0 ) 
		colClass[i] = java.sql.Timestamp.class;
	    else 
		colClass[i] = Object.class;

	    //System.out.println(" column:" + i + " class:" + colClass[i].getName() );
	    
	}
    }

    public void setPrimaryKey(String[] pkeys) {
	pkColumns = pkeys;
	
	ispkColumn = new boolean[ ezt.getColumnCount() ];
	for (int i=0; i < ezt.getColumnCount(); i++) {
	    ispkColumn[i] = false;
	    for (int j=0; j < pkeys.length; j++) {
		if (pkeys[j].compareTo( ezt.getColumnName(i) ) == 0) 
		    ispkColumn[i] = true;
	    }
	}

	//debugPrimaryKeys("setPrimaryKey");
    }

    void debugPrimaryKeys(String where) {
	System.out.println("Debugging primary keys at: " + where);
	System.out.println("pkColumns count: " + pkColumns.length);
	System.out.println("ezt Column count:" + ezt.getColumnCount()); 
	for (int i=0; i < ezt.getColumnCount(); i++) {
	    if (ispkColumn[i]) 
		System.out.println("Primary is : " + ezt.getColumnName(i));
	    else 
		System.out.println("Primary is not: " + ezt.getColumnName(i));
	    
	}
	System.out.println("Finished debugging.");
    }

    public boolean isPrimaryKey(int column) {
	if (ispkColumn == null || ispkColumn.length <= column) 
	    return false;
	return ispkColumn[column];
    }

    public void setPrimaryTable(String tname) {
	pkTable = tname;
    }

    public void set_identity_column (int i) {
	identity_column = i;
    }

    public Integer getModStatus(int row, int col) {
	try {
	    return (Integer) modified.getValueAt(row, col);
	} catch (Exception ex) {
	    return UNMODIFIED;
	}
    }

    public Integer getModStatus(int row) {
	try {
	    return (Integer) modifiedrow.elementAt(row);
	} catch (Exception ex) {
	    return ROW_UNMODIFIED;
	}
    }

    public boolean isModified() {
	for (int i=0; i < ezt.getRowCount(); i++) {
	    if (getModStatus(i) != ROW_UNMODIFIED)
		return true;
	}
	if (ezt_deleted.getRowCount() > 0)
	    return true;
	
	return false;
    }

    protected void setNotModified() {
    	modified    = new ezTable( ezt.getRowCount(), ezt.getColumnCount());
	modifiedrow = new Vector (ezt.getRowCount());

	for (int i=0; i < ezt.getRowCount(); i++) {
	    for (int j=0; j < ezt.getColumnCount(); j++) {
		modified.setValueAt( UNMODIFIED, i, j);
	    }
	    modifiedrow.add(ROW_UNMODIFIED);
	}
    }
    protected void setNotModified(int row) {
	for (int j=0; j < ezt.getColumnCount(); j++) {
	    modified.setValueAt( UNMODIFIED, row, j);
	}
	modifiedrow.setElementAt(ROW_UNMODIFIED, row);
    }
    protected void setNotModified(int row, int col) {
	modified.setValueAt(UNMODIFIED, row, col);
	boolean modflag = false;
	for (int j = 0; j < ezt.getColumnCount(); j++) {
	    if (modified.getValueAt(row,j) == MODIFIED) {
		modflag = true;
	    }
	}
	if (modifiedrow.elementAt(row) != ROW_INSERTED) {
	    if (modflag ) {
		modifiedrow.setElementAt(ROW_MODIFIED, row);
	    } else {
		modifiedrow.setElementAt(ROW_UNMODIFIED, row);
	    }
	}
    }

    //////////////////////////////////////////////////////////////////////////
    //
    //             Implementation of the TableModel Interface
    //
    //////////////////////////////////////////////////////////////////////////

    // MetaData

    public String getColumnName(int column) {
	return ezt.getColumnName(column);
    }

    public Class getColumnClass(int column) {
	return colClass[column];
    } 
 
    public void setEditable(boolean []ed) {
	editable = ed;
	return;
    }

    boolean []editable;

    public boolean isCellEditable(int row, int column) {
	if (pkTable == null || editable == null) 
	    return false;
	
	return editable[column];

//  	if (ezt.getColTypeName(column).indexOf("binary") >= 0)
//  	    return false;

//  	if (identity_column == column)
//  	    return false;

//  	System.out.println("editable row and column " + row + " " + column
//  			   + " with type " + ezt.getColTypeName(column));
//  	return true;
	// return (isPrimaryKey(column) ? false : true );
	//        try {
	//            return colModifiable[column];
	//         }
	//        catch (SQLException e) {
	//            return false;
	//        }
    }

    public int getColumnCount() {
        return ezt.getColumnCount();
    }

    // Data methods

    public int getRowCount() {
        return ezt.getRowCount();
    }

    public Object getValueAt(int arow, int acolumn) {
	return ezt.getValueAt(arow, acolumn); 
    }

    public String dbRepresentation(int column, Object value) {

        if (value == null) {
            return "null";
        }

        Class cc = getColumnClass(column);

//    	System.out.println (value.toString() + " has type " + cc.getName() + "\n");

	if (cc == Boolean.class) 
            return ((Boolean)value).booleanValue() ? "1" : "0";

	if (cc == String.class)
	    return "'"+ quotesafe(value.toString())+"'";

	if (cc == java.sql.Timestamp.class || cc == java.sql.Date.class) {
	    String str1 = value.toString();
	    if (str1.length() > 23)
		str1 = str1.substring(0,23);
	    return "'" + str1 + "'";
	}

	if (Number.class.isAssignableFrom(cc))
	    return value.toString();

// 	System.out.println(value.getClass().getName());
	return "'"+ quotesafe(value.toString())+"'";

    }

    public void setValueAt(Object value, int row, int column) {
        Object orig_val = ezt.getValueAt(row, column);
        if (orig_val == null && (value == null ||
                                 value.toString().length() == 0))
            return;
        if (orig_val != null && value != null)
            if (orig_val.toString().compareTo(value.toString()) == 0)
                return;
        
	ezt.setValueAt(value, row, column);
	modified.setValueAt(MODIFIED, row, column);
	if (modifiedrow.elementAt(row) == ROW_UNMODIFIED) {
	    modifiedrow.setElementAt(ROW_MODIFIED, row);
	    fireTableChanged(new TableModelEvent(this));
	}
    }

    public void update() {
	if (pkTable == null) {
	    System.out.println("Table name returned null.");
	    return;
	}
	boolean errors_happened = true;
	int         error_on_row = -1;
	try {
	    
	    asse.dbq.executeQuery("if (@@trancount = 0) begin tran else raiserror 20001 'Already in transaction. Aborting.'");
	    
	    int rows_to_delete = ezt_deleted.getRowCount();
	    for (int i_row = 0; i_row < rows_to_delete; i_row++) {
		if (pkTable == null) {
		    System.out.println("Table name returned null.");
		    return;
		}

		StringBuffer sqlcmd = new StringBuffer("delete " + pkTable);
		StringBuffer sqlwhere = new StringBuffer("\nwhere\n");
		
		String       whereseparator = "";
		for (int i_col = ezt.getColumnCount() - 1; i_col >= 0; i_col--) {
		    if (ispkColumn[i_col]) {
			sqlwhere.append (whereseparator +
					 ezt.getColumnName(i_col) +
					 " = " +
					 dbRepresentation(i_col, 
							  ezt_deleted.getValueAt(i_row, i_col)));
			whereseparator = " and\n";
		    } 
		}
		sqlcmd.append(sqlwhere);

		asse.dbq.executeQuery(new String(sqlcmd));
		int i = asse.dbq.rowcount();
		if (i > 1) {
		    // We have an error condition.
		    // A single row should have been affected.
		    throw (new SQLException("Multiple rows("+ i + ") deleted by this statement. A bug in DWBJ. But you were most likely not harmed because the transaction was not comitted. Please exit from DWBJ without any further operations. Stop using it till you get an updated version from the Author.:" + sqlcmd));
		} else if (i < 1) {
		    throw (new SQLException("No row affected by the following SQL command. Possibly deleted by other application.:" + sqlcmd + "If you are unsure, please RESET AND START AGAIN"));
		};

	    }

	    error_on_row = -1;
	    for (int i_row=0; i_row < ezt.getRowCount(); i_row++) {
		Integer status = (Integer) modifiedrow.elementAt(i_row);
		error_on_row = i_row;
		if (status == ROW_MODIFIED) 
		    updateDbRow(i_row);
		else if (status == ROW_INSERTED)
		    insertDbRow(i_row);
	    }

	    asse.dbq.executeQuery("commit tran");
	    errors_happened = false;
	}
	catch (SQLException sqle) {
	    try {
		asse.dbq.executeQuery("rollback tran");
	    }
	    catch (SQLException sqle2) {
		sqle2.printStackTrace();
	    }
	    StringBuffer error_msg = new StringBuffer();
	    error_msg.append(sqle.getMessage());
	    if (error_on_row >= 0) {
		error_on_row++;
		error_msg.append(" *** ERROR on ROW : " + error_on_row + " ***" );
	    }
	    ErrDialog ed = new ErrDialog (assemdi_frame.get(), "DATABASE ERROR", 
					  error_msg.toString());
	}
	
	if (errors_happened  == false) {
	    ezt_deleted.setNumRows(0);
	    setNotModified();
	    ezt_orig = new ezTable(ezt);
	}
	
    }
    private void updateDbRow(int i_row) throws SQLException  {
        if (pkTable == null) {
	    System.out.println("Table name returned null.");
	    return;
        }
	if (i_row < 0 || i_row > ezt.getRowCount()) {
	    System.out.println("Invalid Row Count" + i_row);
	    return;
	}

        StringBuffer sqlcmd = new StringBuffer("update " + pkTable);
        StringBuffer sqlwhere = new StringBuffer("\nwhere\n");

	if (modifiedrow.elementAt(i_row) == ROW_MODIFIED) {
	    sqlcmd.append("\nset\n");	
	    String       sqlseparator = " ";
	    String       whereseparator = "";
	    for (int i_col=0; i_col < ezt.getColumnCount(); i_col++) {
		if (modified.getValueAt(i_row, i_col) == MODIFIED) {
		    sqlcmd.append(sqlseparator +
				  ezt.getColumnName(i_col) +
				  " = " +
				  dbRepresentation(i_col, 
						   ezt.getValueAt(i_row, i_col)));
		    sqlseparator = ",\n";
		} 

		if (ispkColumn[i_col]) {
		    sqlwhere.append (whereseparator +
				     ezt.getColumnName(i_col) +
				     " = " +
				     dbRepresentation(i_col, 
						      ezt_orig.getValueAt(i_row, i_col)));
		    whereseparator = " and\n";
		}
	    }
	    sqlcmd.append(sqlwhere);
	    asse.dbq.executeQuery(new String(sqlcmd));
	    int i = asse.dbq.rowcount();
	    if (i > 1) {
		// We have an error condition.
		// A single row should have been affected.
		throw (new SQLException("Multiple rows("+ i + ") updated by this statement. A bug in DWBJ. But you were most likely not harmed since the transaction was not committed. Please exit from DWBJ without any further operations. Stop using it till you get an updated version from the Author.:" + sqlcmd));
	    } else if (i < 1) {
		throw (new SQLException("No row affected by the following SQL command. Possibly deleted by other application:" + sqlcmd + "If you are unsure, please RESET AND START AGAIN"));
	    };
	}
    }
    private void insertDbRow(int i_row) throws  SQLException {
        if (pkTable == null) {
	    System.out.println("Table name returned null.");
	    return;
        }
	if (i_row < 0 || i_row > ezt.getRowCount()) {
	    System.out.println("Invalid Row Count" + i_row);
	    return;
	}
	StringBuffer sqlcmd = new StringBuffer("insert " + pkTable + "\n(" );
	String       sqlseparator = " ";

	for (int i_col=0; i_col < ezt.getColumnCount(); i_col++) {
	    if (modified.getValueAt(i_row, i_col) == MODIFIED) {
		sqlcmd.append(sqlseparator +
			      ezt.getColumnName(i_col) + "\n");
		sqlseparator = ",";
	    }
	}
	sqlcmd.append(") values\n(");
	sqlseparator = " ";
	for (int i_col=0; i_col < ezt.getColumnCount(); i_col++) {
	    if (modified.getValueAt(i_row, i_col) == MODIFIED) {
		sqlcmd.append(sqlseparator +
			      dbRepresentation(i_col, ezt.getValueAt(i_row, i_col)) + 
			      "\n");
		sqlseparator = ",";
	    }
	}
	sqlcmd.append(")");
	asse.dbq.executeQuery(new String(sqlcmd));
	int i =  asse.dbq.rowcount();
	if (i > 1) {
	    // We have an error condition.
	    // A single row should have been affected.
	    throw (new SQLException("Multiple rows("+ i + ") inserted by this statement. A bug in DWBJ. But you were most likely not harmed since the transaction was not committed. Please exit from DWBJ without any further operations. Stop using it till you get an updated version from the Author.:" + sqlcmd));
	} else if (i < 1) {
	    throw (new SQLException("No row affected by the following SQL command ! Possibly a dwbj but. Report to author:" + sqlcmd + "If you are unsure, please RESET AND START AGAIN"));
	};

	// If there was any identity, lets get @@identity value 
	if (identity_column >= 0) {
	    ezTable [] res;
	    res = asse.dbq.executeQuery("select @@identity whatwal");
	    if (res.length < 1) 
		return;

	    Object identity_value = res[0].getValueAt(0, 0);
	    setValueAt(identity_value, i_row, identity_column);
	    setNotModified(i_row, identity_column);
	}

    }

    // Returns the count of rows reset
    //
    public int resetRows(int row_array[]) {
	int i;
	int reset_count = 0;
//  	for (i = 0;
//  	     i < row_array.length;
//  	     i++) {
//  	    if (modifiedrow.elementAt(row_array[i]) != ROW_MODIFIED) {
//  		new ErrDialog(assemdi_frame.get(), "ERROR", 
//  			      "Only modified rows can be reset. Row " + 
//  			      (row_array[i] + 1) + 
//  			      " is not of status modified.");
//  		return;
//  	    }
//  	}
	for (i = 0;
	     i < row_array.length;
	     i++) {
	    reset_count += resetRow(row_array[i]);
	}
	return reset_count;
    }

    private int resetRow(int i_row) {
	// reset a modified row to original state.
	if (modifiedrow.elementAt(i_row) != ROW_MODIFIED) {
//  	    new ErrDialog(assemdi_frame.get(), "ERROR", "Only modified rows can be reset.");
	    return 0;
	}

	for (int j = 0; j < getColumnCount(); j++) {
	    setValueAt(ezt_orig.getValueAt(i_row, j), i_row, j);
	} 

	setNotModified(i_row);
	return 1;
    }

    public void clearAllRows() {
	ezt.setNumRows(0);
	ezt_deleted.setNumRows(0);
	ezt_orig.setNumRows(0);
	setNotModified();

	//fireTableDataChanged();
	fireTableChanged(new TableModelEvent(this) );
    }

    public void setAllData( ezTable pezt ) {

	ezt = pezt;
	ezt_orig = new ezTable(pezt);
	ezt_deleted.setRowCount(0);
	setNotModified();

	//fireTableDataChanged();
	fireTableChanged(new TableModelEvent(this) );

	//debugPrimaryKeys("setAllData");
    }

    // return 1 upon success, -1 upon failure
    public int deleteRows(int row_array[]) {
	int i;
	int start_pos = min_pos (row_array);
	if (start_pos < 0)
	    return -1;
	
	for (i = row_array.length - 1; i >= 0; i--)
	    if (modifiedrow.elementAt(row_array[i]) == ROW_MODIFIED) {
		// can't delete this row
		ErrDialog ed = new ErrDialog(assemdi_frame.get(), "ERROR", "Can't delete modified rows, reset them first.");
		return -1;
	    }
	// We need to go backwards, since otherwise our count
	// will  get screwed after the first delete.

	int del_pos = ezt_deleted.getRowCount();
	for (i = row_array.length - 1; i >= 0; i--) {
	    int i_row = row_array[i];
	    if (modifiedrow.elementAt(i_row) != ROW_INSERTED) {
		// copy the row into ezt_deleted
		Object[] row_data = new Object[ ezt.getColumnCount() ];
		for (int i1 = ezt.getColumnCount() - 1; i1 >=  0; i1--) {
		    row_data[i1] = ezt.getValueAt(i_row, i1);
		}
		ezt_deleted.insertRow(del_pos, row_data );
	    }
	    ezt.removeRow(i_row);
	    ezt_orig.removeRow(i_row);
	    
	    modifiedrow.removeElementAt(i_row);
	    modified.removeRow(i_row);
	}

	return 1;
    }
	
//      public  int deleteRow(int i_row) {

//  	if (modifiedrow.elementAt(i_row) == ROW_MODIFIED) {
//  	    // can't delete this row
//  	    ErrDialog ed = new ErrDialog(assemdi_frame.get(), "ERROR", "Can't delete a modified row, reset it first.");
//  	    return -1;
//  	}

//  	if (modifiedrow.elementAt(i_row) != ROW_INSERTED) {
//  	    // copy the row into ezt_deleted
//  	    Object[] row_data = new Object[ ezt.getColumnCount() ];
//  	    for (int i = ezt.getColumnCount(); i >  0; i--) {
//  		row_data[i - 1] = ezt.getValueAt(i_row, i - 1);
//  	    }
//  	    ezt_deleted.addRow( row_data );
//  	}
//  	ezt.removeRow(i_row);
//  	ezt_orig.removeRow(i_row);

//  	modifiedrow.removeElementAt(i_row);
//  	modified.removeRow(i_row);

//  	this.fireTableChanged(
//  			      new TableModelEvent(ezt)); 
//  	return 1;
//      }

    public void insertRow(int x) {
	int row_arr[] = new int[1];
	row_arr[0] = x;
	insertRows(row_arr);
    }

    // returns number of rows inserted
    public int insertRows(int row_arr[], boolean flag_dup ) {

        if (row_arr.length == 0) {
	    if (ezt.getRowCount() == 0) {
		row_arr = new int[1];
		row_arr[0] = 0;
	    } else
		return 0;
	}

	/////////////////////////
	// We have to traverse from the highest index to lowest
	// to be correct
	/////////////////////////
	int new_pos;
	int start_scan = row_arr.length - 1;
        
        while (start_scan >= 0) {
            
            int band_start = row_arr[start_scan];
            int band_end   = row_arr[start_scan];
        
            int i;
            for (i = start_scan - 1; i >= 0; i--) {
                if ( row_arr[i] + 1 != row_arr[i + 1] )
                    break;
                band_start = row_arr[i];
            }
            start_scan = i;
            
            for (int cur_pos = band_end; cur_pos >= band_start; cur_pos--) {
                
                Object [] values;
                values = new Object[ezt.getColumnCount()];
                
                Object [] modified_status;
                modified_status = new Object [ ezt.getColumnCount() ];
                
                if (flag_dup) {
                    new_pos = band_end + 1;
                    for (int j = 0; j < values.length; j++) {
                        values[j] = ezt.getValueAt(cur_pos, j);
                        modified_status [j] = (identity_column == j ? UNMODIFIED : MODIFIED);
                    }
                } else {
                    new_pos = band_start;
                    for (int j = 0; j < values.length; j++) {
                        modified_status [j] = UNMODIFIED;
                    }
                }
                
                ezt.insertRow(new_pos, values);
                modifiedrow.insertElementAt(ROW_INSERTED, new_pos);
                modified.insertRow(new_pos, modified_status);
                // we are just using array arr below to keep
                // the rowcounts the same so that the current and ezt_orig
                // rows remain matched.
                ezt_orig.insertRow(new_pos, values);
            }
	}
	this.fireTableChanged(
			      new TableModelEvent(ezt)); 

	return row_arr.length;
    }

    public void insertRows(int row_arr[]) {
	
	boolean flag_dup = false;
	insertRows(row_arr, flag_dup);

    }

    public String get_status (int selected_row) {
	int modcount = 0;
	int inscount = 0;
	int delcount = 0;
	Integer stat;
	String row_stat = "***";

	for ( int i = 0; i < modifiedrow.size(); i++) {
	    stat = (Integer)modifiedrow.get(i);
	    if (stat == ROW_INSERTED) 
		inscount++;
	    else if (stat == ROW_MODIFIED) 
		modcount++;
	}

	if (selected_row >= 0) {
	    stat = (Integer)modifiedrow.get(selected_row);
	    row_stat = "UNMODIFIED";
	    if (stat == ROW_INSERTED) 
		row_stat = "NEW";
	    else if (stat == ROW_MODIFIED) 
		row_stat = "MODIFIED";
	}
	delcount = ezt_deleted.getRowCount();

	return (new String( selected_row + 1 + "/" + ezt.getRowCount() +
			    " CURRENT:" + row_stat + ",  TOTAL= NEW: " + inscount +
			    ", MODIFIED: " + modcount +
			    ", DELETED: " + delcount ) ) ;
    }

    public AbstractTableModel get_deleted() {

	Vector colnames = new Vector();
	for (int i =  0; i < ezt.getColumnCount(); i++) {
	    colnames.add( ezt.getColumnName(i) );
	}
	return new DefaultTableModel ( ezt_deleted.getDataVector(), colnames);
    }
    public boolean matchesTable(TableInfo ti) {
	try {
	    if (ti == null) {
		System.out.println("passed ti is null.");
		return false;
	    }
	    String index_names[] = ti.get_uniqueIndexes();
	    if (index_names.length == 0) {
		System.out.println("Target does not have indexes.!");
		return false;
	    }

	    if (getColumnCount() != ti.get_columnCount()) {
		System.out.println("column counts do not match.");
		return false;
	    }
	    for (int i = 0; i <  getColumnCount(); i++) {
		if (getColumnName(i).compareTo(ti.get_columnName(i + 1).trim()) != 0) {
		    System.out.println("Column Name: '" + 
				       getColumnName(i) +  
				       "' does not match '" +
				       ti.get_columnName(i + 1).trim() +
				       "'");
		    ti.print();
		    return false;
		}
	    }
	    return true;
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	    return false;
	}
    }

    // We add a row with state not modified.

    public void reAddRow ( Vector rowdata ) {
	int inserted_row_id = this.getRowCount();
	ezt.addRow(rowdata);
	ezt_orig.addRow((Vector)(rowdata.clone()));

	modifiedrow.add(inserted_row_id,ROW_UNMODIFIED);
	
	Object statuses[] = new Object [ getColumnCount() ];
	for (int i = 0; i < statuses.length; i++ ) 
	    statuses[i] = UNMODIFIED;
	modified.addRow(statuses);

	this.setNotModified(inserted_row_id);
	this.fireTableRowsInserted(inserted_row_id, inserted_row_id);
    }

	
    public boolean matches(ezTable othert) {
	try {
	    if (othert == null) {
		System.out.println("The other table is null.");
		return false;
	    }
	    if (getColumnCount() != othert.getColumnCount()) {
		System.out.println("column counts do not match.");
		return false;
	    }
	    for (int i = getColumnCount() - 1; i >= 0; i-- ) {
		if (ezt.getColTypeName(i).compareTo(othert.getColTypeName(i)) != 0)
		    return false;
	    }
	    return true;
	}
	catch (Exception ex) {
	    return false;
	}
    }

    private int min_pos ( int row_arr[] ) {
	if (row_arr.length == 0) 
	    return -1;
	if (row_arr[0] <= row_arr[row_arr.length - 1]) 
	    return row_arr[0];
	else
	    return row_arr[row_arr.length - 1];
    }
    
    private String quotesafe (String inval) {

	if (inval.indexOf('\'') < 0) 
	    return inval;
	
	StringTokenizer st = new StringTokenizer(inval, "'");

	String outval = "";
	String sep = "";
	while (st.hasMoreTokens()) {
	    outval = outval + sep + st.nextToken();
	    sep = "''";
	}

	return outval;
    }
    
     public void sort (int Column, boolean Ascending) {
         if (isModified())
             return;
        
         ezt.sort(Column, Ascending);
         setOtherezTables();
     }
     
     private void setOtherezTables() {
         ezt_orig = new ezTable(ezt);
         ezt_deleted = new ezTable( 0, ezt.getColumnCount() );
     }
}

